@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2013. Victor M. Alvarez [plusvic@gmail.com].
+Copyright (c) 2013. The YARA Authors. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@ limitations under the License.
 */
 
 
-#ifndef _ATOMS_H
-#define _ATOMS_H
+#ifndef YR_ATOMS_H
+#define YR_ATOMS_H
 
 #include "re.h"
 #include "yara.h"
