@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2007. Victor M. Alvarez [plusvic@gmail.com].
+Copyright (c) 2007-2013. The YARA Authors. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -67,7 +67,8 @@ PIMAGE_NT_HEADERS yr_get_pe_header(
   headers_size += pe_header->FileHeader.SizeOfOptionalHeader;
 
   if (pe_header->Signature == IMAGE_NT_SIGNATURE &&
-      pe_header->FileHeader.Machine == IMAGE_FILE_MACHINE_I386 &&
+      (pe_header->FileHeader.Machine == IMAGE_FILE_MACHINE_I386 ||
+       pe_header->FileHeader.Machine == IMAGE_FILE_MACHINE_AMD64) &&
       buffer_length > headers_size)
   {
     return pe_header;
@@ -86,8 +87,11 @@ uint64_t yr_pe_rva_to_offset(
 {
   int i = 0;
   PIMAGE_SECTION_HEADER section;
+  DWORD section_rva;
+  DWORD section_offset;
 
   section = IMAGE_FIRST_SECTION(pe_header);
+  section_rva = 0;
 
   while(i < MIN(pe_header->FileHeader.NumberOfSections, 60))
   {
@@ -95,9 +99,10 @@ uint64_t yr_pe_rva_to_offset(
         (uint8_t*) pe_header + sizeof(IMAGE_SECTION_HEADER) < buffer_length)
     {
       if (rva >= section->VirtualAddress &&
-          rva <  section->VirtualAddress + section->SizeOfRawData)
+          section_rva <= section->VirtualAddress)
       {
-        return section->PointerToRawData + (rva - section->VirtualAddress);
+        section_rva = section->VirtualAddress;
+        section_offset = section->PointerToRawData;
       }
 
       section++;
@@ -105,11 +110,11 @@ uint64_t yr_pe_rva_to_offset(
     }
     else
     {
-      break;
+      return 0;
     }
   }
 
-  return 0;
+  return section_offset + (rva - section_rva);
 }
 
 
@@ -147,8 +152,16 @@ uint64_t yr_elf_rva_to_offset_32(
     return 0;
 
   // check to prevent integer wraps
-  if(ULONG_MAX - elf_header->sh_entry_count <
-     sizeof(elf32_section_header_t) * elf_header->sh_entry_count)
+
+  if (ULONG_MAX - elf_header->sh_entry_count <
+      sizeof(elf32_section_header_t) * elf_header->sh_entry_count)
+    return 0;
+
+  // check that 'sh_offset' doesn't wrap when added to the
+  // size of entries.
+
+  if (ULONG_MAX - elf_header->sh_offset <
+      sizeof(elf32_section_header_t) * elf_header->sh_entry_count)
     return 0;
 
   if (elf_header->sh_offset + \
@@ -167,6 +180,7 @@ uint64_t yr_elf_rva_to_offset_32(
         rva <  section->addr + section->size)
     {
       // prevent integer wrapping with the return value
+
       if (ULONG_MAX - section->offset < (rva - section->addr))
         return 0;
       else
@@ -192,6 +206,12 @@ uint64_t yr_elf_rva_to_offset_64(
   if (elf_header->sh_offset == 0 || elf_header->sh_entry_count == 0)
     return 0;
 
+  // check that 'sh_offset' doesn't wrap when added to the
+  // size of entries.
+  if(ULONG_MAX - elf_header->sh_offset <
+     sizeof(elf64_section_header_t) * elf_header->sh_entry_count)
+    return 0;
+
   if (elf_header->sh_offset + \
       sizeof(elf64_section_header_t) * \
       elf_header->sh_entry_count > buffer_length)
