@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2007. Victor M. Alvarez [plusvic@gmail.com].
+Copyright (c) 2007. The YARA Authors. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -14,6 +14,9 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
+#ifndef YR_FILEMAP_H
+#define YR_FILEMAP_H
+
 #ifdef WIN32
 #include <windows.h>
 #define FILE_DESCRIPTOR         HANDLE
@@ -43,3 +46,6 @@ int yr_filemap_map(
 
 void yr_filemap_unmap(
     MAPPED_FILE* pmapped_file);
+
+#endif
+
