@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2007. Victor M. Alvarez [plusvic@gmail.com].
+Copyright (c) 2013. The YARA Authors. All Rights Reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -190,9 +190,21 @@ int yr_hash_table_add(
   }
 
   if (ns != NULL)
+  {
     entry->ns = yr_strdup(ns);
+
+    if (entry->ns == NULL)
+    {
+      yr_free(entry->key);
+      yr_free(entry);
+
+      return ERROR_INSUFICIENT_MEMORY;
+    }
+  }
   else
+  {
     entry->ns = NULL;
+  }
 
   entry->value = value;
   bucket_index = hash(0, (uint8_t*) key, strlen(key));
